@@ -12,12 +12,23 @@ import os
 import threading
 import time
 import uuid
+from dataclasses import dataclass
 from datetime import datetime, timezone
 from pathlib import Path
-from typing import Any, Dict, List, Optional
+from typing import Any, Dict, List, Literal, Optional
 
 import anyio
 
+try:  # pragma: no cover - imported for typing/runtime compatibility only
+    from fastmcp import Context
+except ImportError:  # pragma: no cover
+    try:
+        from mcp.server.fastmcp import (
+            Context,  # type: ignore[import-untyped,assignment]
+        )
+    except ImportError:  # pragma: no cover
+        Context = Any  # type: ignore[misc,assignment]
+
 from igloo_mcp.cache import QueryResultCache
 from igloo_mcp.config import Config
 from igloo_mcp.logging import (
@@ -33,6 +44,7 @@ from igloo_mcp.path_utils import (
     find_repo_root,
     resolve_artifact_root,
 )
+from igloo_mcp.post_query_insights import build_default_insights
 from igloo_mcp.service_layer import QueryService
 from igloo_mcp.session_utils import (
     apply_session_context,
@@ -40,6 +52,7 @@ from igloo_mcp.session_utils import (
     restore_session_context,
     snapshot_session,
 )
+from igloo_mcp.sql_objects import extract_query_objects
 from igloo_mcp.sql_validation import validate_sql_statement
 
 from .base import MCPTool
@@ -79,9 +92,29 @@ def _relative_sql_path(repo_root: Path, artifact_path: Optional[Path]) -> Option
         return artifact_path.resolve().as_posix()
 
 
+@dataclass
+class AsyncQueryJobState:
+    execution_id: str
+    status: Literal["pending", "running", "success", "error"]
+    submitted_ts: float
+    timeout_seconds: int
+    statement_preview: str
+    sql_sha256: str | None = None
+    started_ts: float | None = None
+    completed_ts: float | None = None
+    result: Dict[str, Any] | None = None
+    error: str | None = None
+
+
 class ExecuteQueryTool(MCPTool):
     """MCP tool for executing SQL queries against Snowflake."""
 
+    AUTO_ASYNC_MARGIN_SECONDS = 5.0
+    AUTO_ASYNC_POLL_INTERVAL_SECONDS = 0.5
+    ASYNC_JOB_RETENTION_SECONDS = 600
+    ASYNC_JOB_MAX_ENTRIES = 200
+    ASYNC_JOB_TIMEOUT_GRACE_SECONDS = 5.0
+
     def __init__(
         self,
         config: Config,
@@ -112,6 +145,9 @@ class ExecuteQueryTool(MCPTool):
         self._cache_enabled = self.cache.enabled
         self._cache_mode = self.cache.mode
         self._static_audit_warnings.extend(self.cache.pop_warnings())
+        self._async_jobs: dict[str, AsyncQueryJobState] = {}
+        self._jobs_lock = threading.Lock()
+        self._rpc_soft_timeout = self._resolve_rpc_soft_timeout()
 
     @property
     def name(self) -> str:
@@ -151,6 +187,21 @@ class ExecuteQueryTool(MCPTool):
         )
         return None, warnings
 
+    def _resolve_rpc_soft_timeout(self) -> int:
+        raw = os.environ.get("IGLOO_MCP_RPC_SOFT_TIMEOUT")
+        default = 110
+        if not raw:
+            return default
+        try:
+            parsed = int(raw)
+            # Keep the limit reasonable to avoid accidental 0/negative values
+            return max(parsed, 30)
+        except (TypeError, ValueError):
+            self._static_audit_warnings.append(
+                "Invalid IGLOO_MCP_RPC_SOFT_TIMEOUT value; falling back to 110s."
+            )
+            return default
+
     def _persist_sql_artifact(self, sql_sha256: str, statement: str) -> Optional[Path]:
         if self._artifact_root is None:
             self._transient_audit_warnings.append(
@@ -164,6 +215,295 @@ class ExecuteQueryTool(MCPTool):
             )
         return artifact_path
 
+    async def _enqueue_async_job(
+        self,
+        *,
+        statement: str,
+        warehouse: Optional[str],
+        database: Optional[str],
+        schema: Optional[str],
+        role: Optional[str],
+        timeout_seconds: Optional[int],
+        verbose_errors: bool,
+        reason: Optional[str],
+        normalized_insight: Optional[Insight],
+        ctx: Context | None,
+    ) -> Dict[str, Any]:
+        execution_id = uuid.uuid4().hex
+        sql_sha256 = hashlib.sha256(statement.encode("utf-8")).hexdigest()
+        timeout = timeout_seconds or getattr(self.config, "timeout_seconds", 120)
+        now = time.time()
+        job_state = AsyncQueryJobState(
+            execution_id=execution_id,
+            status="pending",
+            submitted_ts=now,
+            timeout_seconds=timeout,
+            statement_preview=statement[:200],
+            sql_sha256=sql_sha256,
+        )
+        with self._jobs_lock:
+            self._prune_async_jobs_locked(now)
+            self._async_jobs[execution_id] = job_state
+
+        if ctx is not None:
+            try:
+                await ctx.report_progress(0, 100, "Query enqueued for async execution")
+            except Exception:
+                pass
+
+        async def _async_runner() -> None:
+            await self._run_async_job(
+                job_state,
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                sql_sha_override=sql_sha256,
+            )
+
+        def _runner() -> None:
+            anyio.run(_async_runner)
+
+        threading.Thread(target=_runner, daemon=True).start()
+
+        return {
+            "status": "accepted",
+            "execution_id": execution_id,
+            "message": (
+                "Query accepted for asynchronous execution. Call fetch_async_query_result"
+                " with this execution_id to check status or retrieve cached rows."
+            ),
+            "timeout_seconds": timeout,
+            "sql_sha256": sql_sha256,
+            "poll_tool": "fetch_async_query_result",
+        }
+
+    async def _execute_auto_mode(
+        self,
+        *,
+        statement: str,
+        warehouse: Optional[str],
+        database: Optional[str],
+        schema: Optional[str],
+        role: Optional[str],
+        timeout_seconds: Optional[int],
+        verbose_errors: bool,
+        reason: Optional[str],
+        normalized_insight: Optional[Insight],
+        ctx: Context | None,
+    ) -> Dict[str, Any]:
+        accepted = await self._enqueue_async_job(
+            statement=statement,
+            warehouse=warehouse,
+            database=database,
+            schema=schema,
+            role=role,
+            timeout_seconds=timeout_seconds,
+            verbose_errors=verbose_errors,
+            reason=reason,
+            normalized_insight=normalized_insight,
+            ctx=ctx,
+        )
+
+        execution_id = accepted.get("execution_id")
+        if not execution_id:
+            return accepted
+
+        inline_wait = self._auto_inline_wait_budget(timeout_seconds)
+        if inline_wait <= 0:
+            accepted["inline_wait_seconds"] = 0.0
+            accepted.setdefault(
+                "message",
+                "Query running asynchronously; call fetch_async_query_result with the "
+                "provided execution_id to retrieve results when complete.",
+            )
+            return accepted
+
+        deadline = time.time() + inline_wait
+        poll_interval = min(
+            self.AUTO_ASYNC_POLL_INTERVAL_SECONDS,
+            max(inline_wait / 5, 0.1),
+        )
+
+        while True:
+            poll = await self.fetch_async_result(
+                execution_id=execution_id,
+                include_rows=True,
+            )
+            status = poll.get("status")
+            if status == "success" and poll.get("result"):
+                return poll["result"]
+            if status == "error":
+                raise RuntimeError(poll.get("error") or "Asynchronous query failed")
+
+            remaining = deadline - time.time()
+            if remaining <= 0:
+                break
+
+            await anyio.sleep(min(poll_interval, remaining))
+
+        # Inline wait exhausted; attempt one more fetch to capture recent completion
+        poll = await self.fetch_async_result(
+            execution_id=execution_id,
+            include_rows=True,
+        )
+        status = poll.get("status")
+        if status == "success" and poll.get("result"):
+            return poll["result"]
+        if status == "error":
+            raise RuntimeError(poll.get("error") or "Asynchronous query failed")
+
+        accepted["inline_wait_seconds"] = inline_wait
+        accepted["message"] = (
+            "Query still running after %.0fs; call fetch_async_query_result "
+            "with execution_id=%s to retrieve cached results."
+            % (
+                inline_wait,
+                execution_id,
+            )
+        )
+        return accepted
+
+    async def _run_async_job(
+        self,
+        job_state: AsyncQueryJobState,
+        *,
+        statement: str,
+        warehouse: Optional[str],
+        database: Optional[str],
+        schema: Optional[str],
+        role: Optional[str],
+        timeout_seconds: Optional[int],
+        verbose_errors: bool,
+        reason: Optional[str],
+        normalized_insight: Optional[Insight],
+        sql_sha_override: Optional[str],
+    ) -> None:
+        with self._jobs_lock:
+            job_state.status = "running"
+            job_state.started_ts = time.time()
+        try:
+            result = await self._execute_impl(
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                ctx=None,
+                execution_id_override=job_state.execution_id,
+                sql_sha_override=sql_sha_override,
+                validate_profile=False,
+                validate_statement=False,
+            )
+            with self._jobs_lock:
+                if job_state.status != "error":
+                    job_state.status = "success"
+                    job_state.result = result
+                    job_state.completed_ts = time.time()
+        except Exception as exc:
+            with self._jobs_lock:
+                if job_state.status != "error":
+                    job_state.status = "error"
+                    job_state.error = str(exc)
+                    job_state.completed_ts = time.time()
+
+    async def fetch_async_result(
+        self,
+        *,
+        execution_id: str,
+        include_rows: bool = True,
+    ) -> Dict[str, Any]:
+        self._prune_async_jobs()
+        with self._jobs_lock:
+            job = self._async_jobs.get(execution_id)
+
+        if job is None:
+            raise ValueError(
+                f"No async query execution found for execution_id={execution_id}"
+            )
+
+        payload: Dict[str, Any] = {
+            "execution_id": execution_id,
+            "status": job.status,
+            "submitted_ts": job.submitted_ts,
+            "started_ts": job.started_ts,
+            "completed_ts": job.completed_ts,
+            "timeout_seconds": job.timeout_seconds,
+            "sql_sha256": job.sql_sha256,
+            "statement_preview": job.statement_preview,
+        }
+
+        cleanup_after_response = False
+        if job.status == "success" and job.result is not None:
+            result = job.result
+            if not include_rows and "rows" in result:
+                result = dict(result)
+                result.pop("rows", None)
+            payload["result"] = result
+            cleanup_after_response = include_rows
+        elif job.status == "error":
+            payload["error"] = job.error
+            cleanup_after_response = True
+
+        if cleanup_after_response:
+            self._cleanup_async_job(execution_id)
+
+        return payload
+
+    def _cleanup_async_job(self, execution_id: str) -> None:
+        with self._jobs_lock:
+            self._async_jobs.pop(execution_id, None)
+
+    def _prune_async_jobs(self) -> None:
+        with self._jobs_lock:
+            self._prune_async_jobs_locked()
+
+    def _prune_async_jobs_locked(self, now: Optional[float] = None) -> None:
+        if now is None:
+            now = time.time()
+
+        to_remove: list[str] = []
+        for exec_id, job in list(self._async_jobs.items()):
+            if (
+                job.completed_ts is not None
+                and now - job.completed_ts > self.ASYNC_JOB_RETENTION_SECONDS
+            ):
+                to_remove.append(exec_id)
+
+        for exec_id in to_remove:
+            self._async_jobs.pop(exec_id, None)
+
+        # Mark long-running in-flight jobs as timed out so callers see terminal state
+        for job in self._async_jobs.values():
+            if job.status in {"pending", "running"} and job.timeout_seconds:
+                if now - job.submitted_ts > (
+                    job.timeout_seconds + self.ASYNC_JOB_TIMEOUT_GRACE_SECONDS
+                ):
+                    job.status = "error"
+                    job.error = (
+                        f"Timed out after {job.timeout_seconds}s of asynchronous work"
+                    )
+                    job.completed_ts = now
+
+        excess = len(self._async_jobs) - self.ASYNC_JOB_MAX_ENTRIES
+        if excess > 0:
+            for exec_id, _ in sorted(
+                self._async_jobs.items(), key=lambda item: item[1].submitted_ts
+            ):
+                if excess <= 0:
+                    break
+                self._async_jobs.pop(exec_id, None)
+                excess -= 1
+
     def _resolve_cache_context(
         self, overrides: Dict[str, Optional[str]]
     ) -> tuple[Dict[str, Optional[str]], bool]:
@@ -218,6 +558,78 @@ class ExecuteQueryTool(MCPTool):
         warnings.extend(self.cache.pop_warnings())
         return warnings
 
+    def _auto_inline_wait_budget(self, timeout_seconds: Optional[int]) -> float:
+        user_limit = timeout_seconds or getattr(self.config, "timeout_seconds", 120)
+        rpc_budget = max(
+            float(self._rpc_soft_timeout) - self.AUTO_ASYNC_MARGIN_SECONDS,
+            0.0,
+        )
+        return max(min(float(user_limit), rpc_budget), 0.0)
+
+    async def _ensure_profile_health(self) -> None:
+        if not self.health_monitor:
+            return
+
+        profile_health = await anyio.to_thread.run_sync(
+            self.health_monitor.get_profile_health,
+            self.config.snowflake.profile,
+            False,
+        )
+        if profile_health.is_valid:
+            return
+
+        error_msg = profile_health.validation_error or "Profile validation failed"
+        available = (
+            ", ".join(profile_health.available_profiles)
+            if profile_health.available_profiles
+            else "none"
+        )
+        self.health_monitor.record_error(f"Profile validation failed: {error_msg}")
+        raise ValueError(
+            f"Snowflake profile validation failed: {error_msg}. "
+            f"Profile: {self.config.snowflake.profile}, "
+            f"Available profiles: {available}. "
+            "Check configuration with 'snow connection list' or verify profile settings."
+        )
+
+    def _enforce_sql_permissions(self, statement: str) -> None:
+        allow_list = self.config.sql_permissions.get_allow_list()
+        disallow_list = self.config.sql_permissions.get_disallow_list()
+
+        stmt_type, is_valid, error_msg = validate_sql_statement(
+            statement, allow_list, disallow_list
+        )
+
+        if not is_valid and error_msg:
+            if self.health_monitor:
+                self.health_monitor.record_error(
+                    f"SQL statement blocked: {stmt_type} - {statement[:100]}"
+                )
+            raise ValueError(error_msg)
+
+    def _ensure_default_insights(
+        self, result: Dict[str, Any]
+    ) -> tuple[Optional[Dict[str, Any]], List[str]]:
+        key_metrics = result.get("key_metrics")
+        insights = result.get("insights")
+        rows = result.get("rows")
+
+        if (key_metrics is None or insights is None) and rows:
+            computed_metrics, computed_insights = build_default_insights(
+                rows,
+                columns=result.get("columns"),
+                total_rows=result.get("rowcount"),
+                truncated=bool(result.get("truncated")),
+            )
+            if key_metrics is None and computed_metrics:
+                result["key_metrics"] = computed_metrics
+                key_metrics = computed_metrics
+            if insights is None and computed_insights:
+                result["insights"] = computed_insights
+                insights = computed_insights
+
+        return key_metrics, insights or []
+
     @staticmethod
     def _iso_timestamp(epoch: float) -> str:
         return datetime.fromtimestamp(epoch, tz=timezone.utc).isoformat()
@@ -300,9 +712,21 @@ class ExecuteQueryTool(MCPTool):
                     "timeout_seconds": 120,
                 },
             },
+            {
+                "description": "Run long analytics query asynchronously",
+                "parameters": {
+                    "statement": (
+                        "WITH params AS (SELECT DATEADD('day', -30, CURRENT_DATE) AS start_dt) "
+                        "SELECT * FROM ANALYTICS.LONG_RUNNING_METRICS WHERE event_ts >= (SELECT start_dt FROM params)"
+                    ),
+                    "warehouse": "ANALYTICS_WH",
+                    "timeout_seconds": 480,
+                    "response_mode": "async",
+                },
+            },
         ]
 
-    async def execute(
+    async def _execute_impl(
         self,
         statement: str,
         warehouse: Optional[str] = None,
@@ -312,93 +736,21 @@ class ExecuteQueryTool(MCPTool):
         timeout_seconds: Optional[int] = None,
         verbose_errors: bool = False,
         reason: Optional[str] = None,
-        post_query_insight: Optional[Dict[str, Any] | str] = None,
-        **kwargs: Any,
+        normalized_insight: Optional[Insight] = None,
+        ctx: Context | None = None,
+        *,
+        execution_id_override: Optional[str] = None,
+        sql_sha_override: Optional[str] = None,
+        validate_profile: bool = True,
+        validate_statement: bool = True,
     ) -> Dict[str, Any]:
-        """Execute SQL query against Snowflake.
+        """Internal execute_query implementation shared by sync + async flows."""
 
-        Args:
-            statement: SQL statement to execute
-            warehouse: Optional warehouse override
-            database: Optional database override
-            schema: Optional schema override
-            role: Optional role override
-            timeout_seconds: Query timeout in seconds (default: 30s)
-            verbose_errors: Include detailed optimization hints in errors
-            reason: Short reason for executing this query
-            post_query_insight: Optional dict or string with key findings and metrics from query results
+        if validate_profile:
+            await self._ensure_profile_health()
 
-        Returns:
-            Query results with rows, rowcount, and execution metadata
-
-        Raises:
-            ValueError: If profile validation fails or SQL is blocked
-            RuntimeError: If query execution fails
-        """
-        # Normalize insight once
-        if "metric_insight" in kwargs:
-            raise TypeError(
-                "execute_query no longer accepts 'metric_insight'; use 'post_query_insight' instead"
-            )
-
-        normalized_insight: Optional[Insight] = None
-        if post_query_insight is not None:
-            normalized_insight = normalize_insight(post_query_insight)
-        if timeout_seconds is not None:
-            if isinstance(timeout_seconds, str):
-                try:
-                    timeout_seconds = int(timeout_seconds)
-                except (TypeError, ValueError):
-                    raise TypeError(
-                        "timeout_seconds must be an integer value in seconds."
-                    ) from None
-            if isinstance(timeout_seconds, bool) or not isinstance(
-                timeout_seconds, int
-            ):
-                raise TypeError("timeout_seconds must be an integer value in seconds.")
-            if not 1 <= timeout_seconds <= 3600:
-                raise ValueError("timeout_seconds must be between 1 and 3600 seconds.")
-
-        # Validate profile health before executing query
-        if self.health_monitor:
-            profile_health = await anyio.to_thread.run_sync(
-                self.health_monitor.get_profile_health,
-                self.config.snowflake.profile,
-                False,  # use cache
-            )
-            if not profile_health.is_valid:
-                error_msg = (
-                    profile_health.validation_error or "Profile validation failed"
-                )
-                available = (
-                    ", ".join(profile_health.available_profiles)
-                    if profile_health.available_profiles
-                    else "none"
-                )
-                self.health_monitor.record_error(
-                    f"Profile validation failed: {error_msg}"
-                )
-                raise ValueError(
-                    f"Snowflake profile validation failed: {error_msg}. "
-                    f"Profile: {self.config.snowflake.profile}, "
-                    f"Available profiles: {available}. "
-                    f"Check configuration with 'snow connection list' or verify profile settings."
-                )
-
-        # Validate SQL statement against permissions
-        allow_list = self.config.sql_permissions.get_allow_list()
-        disallow_list = self.config.sql_permissions.get_disallow_list()
-
-        stmt_type, is_valid, error_msg = validate_sql_statement(
-            statement, allow_list, disallow_list
-        )
-
-        if not is_valid and error_msg:
-            if self.health_monitor:
-                self.health_monitor.record_error(
-                    f"SQL statement blocked: {stmt_type} - {statement[:100]}"
-                )
-            raise ValueError(error_msg)
+        if validate_statement:
+            self._enforce_sql_permissions(statement)
 
         # Prepare session context overrides
         overrides_input = {
@@ -421,9 +773,12 @@ class ExecuteQueryTool(MCPTool):
                 "role": overrides_input.get("role"),
             }
 
-        execution_id = uuid.uuid4().hex
+        execution_id = execution_id_override or uuid.uuid4().hex
         requested_ts = time.time()
-        sql_sha256 = hashlib.sha256(statement.encode("utf-8")).hexdigest()
+        sql_sha256 = (
+            sql_sha_override or hashlib.sha256(statement.encode("utf-8")).hexdigest()
+        )
+        referenced_objects = extract_query_objects(statement)
         history_artifacts: Dict[str, str] = {}
         artifact_path = self._persist_sql_artifact(sql_sha256, statement)
         if artifact_path is not None:
@@ -492,12 +847,21 @@ class ExecuteQueryTool(MCPTool):
                 )
             if cache_hit_metadata.get("truncated"):
                 result["truncated"] = cache_hit_metadata.get("truncated")
+            session_context = effective_context.copy()
             if cache_hit_metadata.get("context"):
-                result.setdefault("session_context", cache_hit_metadata["context"])
-            else:
-                result.setdefault("session_context", effective_context)
+                ctx = cache_hit_metadata["context"]
+                session_context.update(
+                    {
+                        k: ctx.get(k)
+                        for k in ["warehouse", "database", "schema", "role"]
+                        if ctx.get(k)
+                    }
+                )
+            result["session_context"] = session_context
             if cache_hit_metadata.get("columns"):
                 result["columns"] = cache_hit_metadata["columns"]
+            if cache_hit_metadata.get("objects"):
+                result["objects"] = cache_hit_metadata["objects"]
 
             # Retrieve stored insight from cache manifest
             stored_insight_raw = cache_hit_metadata.get("post_query_insight")
@@ -510,6 +874,18 @@ class ExecuteQueryTool(MCPTool):
                 )
                 result["post_query_insight"] = stored_insight
 
+            cached_metrics = cache_hit_metadata.get("key_metrics")
+            if cached_metrics:
+                result["key_metrics"] = cached_metrics
+            cached_insights = cache_hit_metadata.get("insights")
+            if cached_insights:
+                result["insights"] = cached_insights
+            cached_objects = cache_hit_metadata.get("objects")
+            if cached_objects:
+                result["objects"] = cached_objects
+
+            key_metrics, derived_insights = self._ensure_default_insights(result)
+
             payload: Dict[str, Any] = {
                 "ts": requested_ts,
                 "timestamp": self._iso_timestamp(requested_ts),
@@ -525,7 +901,15 @@ class ExecuteQueryTool(MCPTool):
                 "cache_manifest": str(cache_hit_metadata.get("manifest_path")),
                 "columns": cache_hit_metadata.get("columns"),
             }
-            payload["session_context"] = effective_context
+            full_session = effective_context.copy()
+            full_session.update(
+                {
+                    k: session_context.get(k)
+                    for k in ["warehouse", "database", "schema", "role"]
+                    if session_context.get(k)
+                }
+            )
+            payload["session_context"] = full_session
             if sql_sha256:
                 payload["sql_sha256"] = sql_sha256
             if history_artifacts:
@@ -542,6 +926,10 @@ class ExecuteQueryTool(MCPTool):
                 payload["post_query_insight"] = truncate_insight_for_storage(
                     stored_insight_for_storage
                 )
+            if key_metrics:
+                payload["key_metrics"] = key_metrics
+            if derived_insights:
+                payload["insights"] = derived_insights
             try:
                 self.history.record(payload)
             except Exception:
@@ -569,6 +957,8 @@ class ExecuteQueryTool(MCPTool):
                 reason,
             )
 
+            key_metrics, derived_insights = self._ensure_default_insights(result)
+
             if self.health_monitor and hasattr(
                 self.health_monitor, "record_query_success"
             ):
@@ -594,6 +984,9 @@ class ExecuteQueryTool(MCPTool):
                         "post_query_insight": cache_insight,
                         "reason": reason,
                         "columns": result.get("columns"),
+                        "key_metrics": key_metrics,
+                        "insights": derived_insights,
+                        "objects": referenced_objects,
                     }
                     manifest_path = self.cache.store(
                         cache_key,
@@ -648,6 +1041,12 @@ class ExecuteQueryTool(MCPTool):
                     payload["cache_manifest"] = str(manifest_path)
                 if result.get("columns"):
                     payload["columns"] = result.get("columns")
+                if key_metrics:
+                    payload["key_metrics"] = key_metrics
+                if derived_insights:
+                    payload["insights"] = derived_insights
+                if referenced_objects:
+                    payload["objects"] = referenced_objects
                 self.history.record(payload)
             except Exception:
                 pass
@@ -663,6 +1062,8 @@ class ExecuteQueryTool(MCPTool):
                 result["cache"]["manifest_path"] = str(manifest_path)
             if session_context:
                 result.setdefault("session_context", session_context)
+            if referenced_objects:
+                result["objects"] = referenced_objects
 
             # Include full (untruncated) insight in response
             if normalized_insight:
@@ -717,6 +1118,8 @@ class ExecuteQueryTool(MCPTool):
                     )
                 if cache_key:
                     payload["cache_key"] = cache_key
+                if referenced_objects:
+                    payload["objects"] = referenced_objects
                 self.history.record(payload)
             except Exception:
                 pass
@@ -795,6 +1198,8 @@ class ExecuteQueryTool(MCPTool):
                     )
                 if cache_key:
                     payload["cache_key"] = cache_key
+                if referenced_objects:
+                    payload["objects"] = referenced_objects
                 self.history.record(payload)
             except Exception:
                 pass
@@ -815,6 +1220,135 @@ class ExecuteQueryTool(MCPTool):
                     f"Query execution failed: {error_message[:150]}. Use verbose_errors=true for details."
                 )
 
+    async def execute(
+        self,
+        statement: str,
+        warehouse: Optional[str] = None,
+        database: Optional[str] = None,
+        schema: Optional[str] = None,
+        role: Optional[str] = None,
+        timeout_seconds: Optional[int] = None,
+        verbose_errors: bool = False,
+        reason: Optional[str] = None,
+        post_query_insight: Optional[Dict[str, Any] | str] = None,
+        response_mode: Optional[str] = None,
+        ctx: Context | None = None,
+        **kwargs: Any,
+    ) -> Dict[str, Any]:
+        """Public entry point for execute_query."""
+
+        if "metric_insight" in kwargs:
+            raise TypeError(
+                "execute_query no longer accepts 'metric_insight'; use 'post_query_insight' instead"
+            )
+
+        normalized_insight: Optional[Insight] = None
+        if post_query_insight is not None:
+            normalized_insight = normalize_insight(post_query_insight)
+
+        if timeout_seconds is not None:
+            if isinstance(timeout_seconds, str):
+                try:
+                    timeout_seconds = int(timeout_seconds)
+                except (TypeError, ValueError):
+                    raise TypeError(
+                        "timeout_seconds must be an integer value in seconds."
+                    ) from None
+            if isinstance(timeout_seconds, bool) or not isinstance(
+                timeout_seconds, int
+            ):
+                raise TypeError("timeout_seconds must be an integer value in seconds.")
+            if not 1 <= timeout_seconds <= 3600:
+                raise ValueError("timeout_seconds must be between 1 and 3600 seconds.")
+
+        # Always validate profile + SQL up front so async paths fail fast.
+        await self._ensure_profile_health()
+        self._enforce_sql_permissions(statement)
+
+        mode = (response_mode or "auto").lower()
+        if mode not in {"auto", "sync", "async"}:
+            raise ValueError("response_mode must be one of: auto, sync, async")
+
+        if mode == "sync":
+            return await self._execute_impl(
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                ctx=ctx,
+            )
+
+        if mode == "async":
+            return await self._enqueue_async_job(
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                ctx=ctx,
+            )
+        # Auto mode chooses between inline execution and async scheduling based
+        # on the available RPC soft timeout budget. When the inline budget can
+        # fully cover the requested timeout we prefer sync execution so that
+        # client-visible behavior (including timeout exceptions) remains
+        # predictable.
+        timeout_limit = timeout_seconds or getattr(self.config, "timeout_seconds", 120)
+        inline_wait = self._auto_inline_wait_budget(timeout_seconds)
+        if inline_wait >= float(timeout_limit):
+            return await self._execute_impl(
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                ctx=ctx,
+            )
+
+        if mode == "auto" and timeout_seconds is not None:
+            inline_budget = self._auto_inline_wait_budget(timeout_seconds)
+            if inline_budget >= float(timeout_seconds):
+                mode = "sync"
+
+        if mode == "sync":
+            return await self._execute_impl(
+                statement=statement,
+                warehouse=warehouse,
+                database=database,
+                schema=schema,
+                role=role,
+                timeout_seconds=timeout_seconds,
+                verbose_errors=verbose_errors,
+                reason=reason,
+                normalized_insight=normalized_insight,
+                ctx=ctx,
+            )
+
+        return await self._execute_auto_mode(
+            statement=statement,
+            warehouse=warehouse,
+            database=database,
+            schema=schema,
+            role=role,
+            timeout_seconds=timeout_seconds,
+            verbose_errors=verbose_errors,
+            reason=reason,
+            normalized_insight=normalized_insight,
+            ctx=ctx,
+        )
+
     def _execute_query_sync(
         self,
         statement: str,
@@ -1161,7 +1695,7 @@ class ExecuteQueryTool(MCPTool):
             "title": "Execute Snowflake Query",
             "type": "object",
             "additionalProperties": False,
-            "required": ["statement"],
+            "required": ["statement", "reason"],
             "properties": {
                 "statement": {
                     **string_schema(
@@ -1181,17 +1715,21 @@ class ExecuteQueryTool(MCPTool):
                 "reason": {
                     **string_schema(
                         (
-                            "Short reason for executing this query. Stored in Snowflake "
-                            "QUERY_TAG, history, and cache metadata to explain why the data was requested. "
-                            "Avoid sensitive information."
+                            "REQUIRED: Short reason for executing this query. "
+                            "Stored in Snowflake QUERY_TAG, history, and cache metadata "
+                            "to explain why the data was requested. Avoid sensitive information. "
+                            "Examples: 'Validate revenue spike', 'Dashboard refresh', 'Debug missing data'"
                         ),
-                        title="Reason",
+                        title="Reason (REQUIRED)",
                         examples=[
                             "Validate yesterday's revenue spike",
                             "Power BI dashboard refresh",
                             "Investigate nulls in customer_email",
+                            "Check Q3 2025 Hyperliquid coverage",
+                            "Explore Base DEX BTC trading volume",
                         ],
                     ),
+                    "minLength": 5,
                 },
                 "warehouse": snowflake_identifier_schema(
                     "Warehouse override. Defaults to the active profile warehouse.",
@@ -1293,5 +1831,17 @@ class ExecuteQueryTool(MCPTool):
                         },
                     ],
                 },
+                "response_mode": {
+                    "title": "Response Mode",
+                    "type": "string",
+                    "enum": ["auto", "sync", "async"],
+                    "default": "auto",
+                    "description": (
+                        "Controls how the tool responds: 'auto' runs synchronously until nearing the client timeout, "
+                        "'async' immediately returns an execution_id for polling via fetch_async_query_result, "
+                        "and 'sync' forces inline execution."
+                    ),
+                    "examples": ["auto", "async"],
+                },
             },
         }
