@@ -14,11 +14,12 @@ import argparse
 import os
 import string
 from contextlib import asynccontextmanager
+from dataclasses import asdict
 from functools import partial
 from typing import Any, Dict, List, Optional
 
 import anyio
-from pydantic import Field
+from pydantic import Field, ValidationError
 from typing_extensions import Annotated
 
 # NOTE: For typing, import from the fastmcp package; fallback handled at runtime.
@@ -62,12 +63,22 @@ from .mcp_health import (
     MCPHealthMonitor,
 )
 from .mcp_resources import MCPResourceManager
-from .path_utils import resolve_artifact_root
+from .path_utils import find_repo_root, resolve_artifact_root, resolve_history_path
 from .profile_utils import (
     ProfileValidationError,
     get_profile_summary,
     validate_and_resolve_profile,
 )
+from .reporting.builder import build_report, lint_report
+from .reporting.history_index import HistoryIndex
+from .reporting.manifest import (
+    DatasetRef,
+    DatasetSource,
+    ReportManifest,
+    ReportOutput,
+    TemplatesConfig,
+    load_manifest,
+)
 from .service_layer import CatalogService, DependencyService, QueryService
 from .session_utils import (
     SessionContext,
@@ -192,6 +203,16 @@ def register_igloo_mcp(
     )
     async def execute_query_tool(
         statement: Annotated[str, Field(description="SQL statement to execute")],
+        reason: Annotated[
+            str,
+            Field(
+                description=(
+                    "Short reason for executing this query. Stored in Snowflake QUERY_TAG "
+                    "and local history; avoid sensitive info."
+                ),
+                min_length=5,
+            ),
+        ],
         warehouse: Annotated[
             Optional[str], Field(description="Warehouse override", default=None)
         ] = None,
@@ -204,16 +225,6 @@ def register_igloo_mcp(
         role: Annotated[
             Optional[str], Field(description="Role override", default=None)
         ] = None,
-        reason: Annotated[
-            Optional[str],
-            Field(
-                description=(
-                    "Short reason for executing this query. Stored in Snowflake QUERY_TAG "
-                    "and local history; avoid sensitive info."
-                ),
-                default=None,
-            ),
-        ] = None,
         timeout_seconds: Annotated[
             Optional[int],
             Field(
@@ -243,6 +254,17 @@ def register_igloo_mcp(
                 default=None,
             ),
         ] = None,
+        response_mode: Annotated[
+            Optional[str],
+            Field(
+                description=(
+                    "Execution response mode: 'auto' (default) runs synchronously until nearing the client timeout, "
+                    "'async' enqueues the query and requires polling via fetch_async_query_result, "
+                    "and 'sync' forces inline execution."
+                ),
+                default=None,
+            ),
+        ] = None,
         ctx: Context | None = None,
     ) -> Dict[str, Any]:
         """Execute a SQL query against Snowflake - delegates to ExecuteQueryTool."""
@@ -257,8 +279,51 @@ def register_igloo_mcp(
                 timeout_seconds=timeout_seconds,
                 verbose_errors=verbose_errors,
                 post_query_insight=post_query_insight,
+                response_mode=response_mode,
                 ctx=ctx,
             )
+        except ValidationError as e:
+            # Enhanced validation error messages for user-friendly feedback
+            errors = e.errors()
+            for err in errors:
+                field = err["loc"][0] if err["loc"] else None
+                error_type = err["type"]
+
+                # Missing reason parameter
+                if field == "reason" and error_type == "missing":
+                    raise ValueError(
+                        "❌ Missing required parameter: 'reason'\n\n"
+                        "The 'reason' parameter is required in v0.2.4+ for query auditability.\n\n"
+                        "💡 Quick fix: Add a brief explanation (5+ characters)\n"
+                        "   execute_query(\n"
+                        "       statement='SELECT * FROM customers LIMIT 10',\n"
+                        "       reason='Sample customer data for debugging'  # ← Add this\n"
+                        "   )\n\n"
+                        "📝 Examples of good reasons:\n"
+                        "   • 'Debug null customer records in Q3'\n"
+                        "   • 'Validate revenue totals for dashboard'\n"
+                        "   • 'Explore product catalog schema'\n\n"
+                        "📚 Docs: https://github.com/Evan-Kim2028/igloo-mcp/blob/main/docs/api/tools/execute_query.md#parameters"
+                    ) from e
+
+                # Reason too short
+                elif field == "reason" and error_type == "string_too_short":
+                    provided = err.get("input", "")
+                    raise ValueError(
+                        f"❌ Parameter 'reason' is too short: '{provided}' ({len(str(provided))} chars)\n\n"
+                        f"Minimum length: 5 characters (you provided {len(str(provided))})\n\n"
+                        "💡 Be more descriptive - explain the query's purpose:\n"
+                        "   ✅ 'Debug sales spike on 2025-01-15'\n"
+                        "   ✅ 'Count active users for monthly report'\n"
+                        "   ✅ 'Explore sales schema for new dashboard'\n"
+                        "   ❌ 'test' (too vague and too short)\n\n"
+                        "📖 Why? Meaningful reasons improve audit trails and help\n"
+                        "   teammates understand query context in history logs."
+                    ) from e
+
+            # Fallback to default Pydantic error for other validation issues
+            raise ValueError(f"Validation error: {e}") from e
+
         except ValueError as e:
             # Enhanced structured error for validation issues
             error_msg = str(e)
@@ -290,8 +355,20 @@ def register_igloo_mcp(
                         },
                     }
                 ) from e
+            elif "SQL statement type" in error_msg and "not permitted" in error_msg:
+                # Enhanced SQL permission error with configuration guidance
+                raise ValueError(
+                    f"{error_msg}\n\n"
+                    "🛡️  Safety Guardrails: This operation is blocked by default.\n\n"
+                    "⚙️  To enable write operations:\n"
+                    "   1. Set environment variable: IGLOO_MCP_SQL_PERMISSIONS='write'\n"
+                    "   2. Or configure in your MCP client settings\n\n"
+                    "⚠️  Warning: Enabling writes removes safety guardrails.\n"
+                    "   Use with caution in production environments.\n\n"
+                    "📚 Docs: https://github.com/Evan-Kim2028/igloo-mcp/blob/main/docs/configuration.md#sql-permissions"
+                ) from e
             elif "SQL statement type" in error_msg:
-                # Pass through enhanced SQL validation errors
+                # Pass through other SQL validation errors
                 raise ValueError(error_msg) from e
             else:
                 # Generic parameter error
@@ -345,6 +422,30 @@ def register_igloo_mcp(
                     }
                 ) from e
 
+    @server.tool(
+        name="fetch_async_query_result",
+        description="Check status or retrieve results for an async execute_query call",
+    )
+    async def fetch_async_query_result_tool(
+        execution_id: Annotated[
+            str,
+            Field(
+                description="Execution ID returned when execute_query runs in async mode",
+            ),
+        ],
+        include_rows: Annotated[
+            bool,
+            Field(
+                description="Include cached rows when the job has finished",
+                default=True,
+            ),
+        ] = True,
+    ) -> Dict[str, Any]:
+        return await execute_query_inst.fetch_async_result(
+            execution_id=execution_id,
+            include_rows=include_rows,
+        )
+
     @server.tool(name="preview_table", description="Preview table contents")
     async def preview_table_tool(
         table_name: Annotated[str, Field(description="Fully qualified table name")],
@@ -505,6 +606,328 @@ def register_igloo_mcp(
             limit=limit,
         )
 
+    @server.tool(
+        name="report_scaffold",
+        description=(
+            "Scaffold a starter report manifest from recent execute_query history so agents "
+            "can edit it and later build reports with report_build."
+        ),
+    )
+    async def report_scaffold_tool(
+        manifest_path: Annotated[
+            Optional[str],
+            Field(
+                description=(
+                    "Optional target path for a manifest YAML (e.g. 'reports/my_report.yaml'). "
+                    "Used to derive report id and default output naming when write_manifest is true. "
+                    "Relative paths are resolved against the repository root."
+                ),
+                default=None,
+            ),
+        ] = None,
+        limit: Annotated[
+            int,
+            Field(
+                description=(
+                    "Number of recent execute_query history entries to turn into datasets in the manifest."
+                ),
+                ge=1,
+                le=20,
+                default=3,
+            ),
+        ] = 3,
+        write_manifest: Annotated[
+            bool,
+            Field(
+                description=(
+                    "If true, also write the scaffolded manifest YAML to disk (using manifest_path or 'report.yaml' "
+                    "in the repo root) and return written_path. When false, only return the manifest JSON payload."
+                ),
+                default=False,
+            ),
+        ] = False,
+    ) -> Dict[str, Any]:
+        """Scaffold a ReportManifest from recent history entries.
+
+        This tool is intended for AI/native workflows: it returns the manifest as
+        structured JSON so agents can edit and commit it, and optionally writes a
+        YAML file when requested.
+        """
+
+        from pathlib import Path
+
+        history_path = resolve_history_path()
+        index = HistoryIndex(history_path)
+        records = index.records
+        records_sorted = sorted(records, key=lambda r: r.get("ts") or 0.0)
+        recent = records_sorted[-limit:] if limit and records_sorted else []
+
+        datasets: List[DatasetRef] = []
+        for idx, record in enumerate(recent, start=1):
+            exec_id = record.get("execution_id")
+            sha = record.get("sql_sha256")
+            artifacts = record.get("artifacts") or {}
+            cache_manifest = artifacts.get("cache_manifest") or record.get(
+                "cache_manifest"
+            )
+            source = DatasetSource(
+                execution_id=str(exec_id) if exec_id else None,
+                sql_sha256=str(sha) if sha else None,
+                cache_manifest=str(cache_manifest) if cache_manifest else None,
+            )
+            description = None
+            if record.get("reason"):
+                description = str(record["reason"])
+            datasets.append(
+                DatasetRef(
+                    name=f"dataset_{idx}",
+                    description=description,
+                    source=source,
+                )
+            )
+
+        if manifest_path is not None:
+            manifest_raw = Path(manifest_path)
+            report_id = manifest_raw.stem or "report"
+        else:
+            report_id = "report"
+
+        manifest = ReportManifest(
+            id=report_id,
+            title=f"Report {report_id}",
+            templates=TemplatesConfig(main="templates/report.md", engine="jinja"),
+            datasets=datasets,
+            outputs=[
+                ReportOutput(
+                    name="default",
+                    format="markdown",
+                    path=f"reports/{report_id}.md",
+                )
+            ],
+        )
+
+        written_path: Optional[str] = None
+        if write_manifest:
+            raw_path = manifest_path or "report.yaml"
+            path = Path(raw_path).expanduser()
+            if not path.is_absolute():
+                repo_root = find_repo_root()
+                path = (repo_root / path).resolve()
+
+            path.parent.mkdir(parents=True, exist_ok=True)
+            try:
+                import yaml  # type: ignore[import-untyped]
+
+                with path.open("w", encoding="utf-8") as fh:
+                    yaml.safe_dump(
+                        manifest.model_dump(by_alias=True, exclude_none=True),
+                        fh,
+                        sort_keys=False,
+                    )
+                written_path = str(path)
+            except Exception as exc:  # pragma: no cover - unlikely I/O error
+                raise RuntimeError(
+                    f"Failed to write scaffolded manifest: {exc}"
+                ) from exc
+
+        return {
+            "manifest": manifest.model_dump(),
+            "report_id": manifest.id,
+            "dataset_count": len(manifest.datasets),
+            "history_path": str(history_path),
+            "written_path": written_path,
+        }
+
+    @server.tool(
+        name="report_build",
+        description=(
+            "Validate a report manifest and build a narrative or JSON payload from cached execute_query results. "
+            "Use validate_only=true to lint without rendering, or default settings to render the report body."
+        ),
+    )
+    async def report_build_tool(
+        manifest_path: Annotated[
+            str,
+            Field(
+                description=(
+                    "Path to the report manifest YAML to use (relative to repo root or absolute), "
+                    "for example 'reports/my_report.yaml'."
+                ),
+                default="report.yaml",
+            ),
+        ] = "report.yaml",
+        output_name: Annotated[
+            Optional[str],
+            Field(
+                description=(
+                    "Logical output name from manifest.outputs to build (default: the first output), "
+                    "for example 'default' or 'html_export'."
+                ),
+                default=None,
+            ),
+        ] = None,
+        format: Annotated[
+            Optional[str],
+            Field(
+                description=(
+                    "Optional override for output format: 'markdown', 'html', or 'json'. "
+                    "Defaults to the manifest output format; 'json' is recommended for agent consumption."
+                ),
+                default=None,
+            ),
+        ] = None,
+        persist_output: Annotated[
+            bool,
+            Field(
+                description=(
+                    "If true, write the rendered body to disk using either output_path "
+                    "or the path defined on the selected manifest.outputs entry."
+                ),
+                default=False,
+            ),
+        ] = False,
+        output_path: Annotated[
+            Optional[str],
+            Field(
+                description=(
+                    "Explicit output path for the rendered report when persist_output is true "
+                    "(relative to repo root or absolute)."
+                ),
+                default=None,
+            ),
+        ] = None,
+        validate_only: Annotated[
+            bool,
+            Field(
+                description=(
+                    "If true, only run validation and return ok/issues without rendering the report body."
+                ),
+                default=False,
+            ),
+        ] = False,
+        fail_on_issues: Annotated[
+            bool,
+            Field(
+                description=(
+                    "If true and validation finds issues, skip rendering and return ok=false with issues, "
+                    "so agents can treat any lint problems as hard failures."
+                ),
+                default=False,
+            ),
+        ] = False,
+        include_body: Annotated[
+            bool,
+            Field(
+                description=(
+                    "If false, omit the rendered body from the response while still returning format, provenance, "
+                    "and any validation issues (useful when agents only need metadata)."
+                ),
+                default=True,
+            ),
+        ] = True,
+    ) -> Dict[str, Any]:
+        import json
+        from pathlib import Path
+
+        raw_path = Path(manifest_path).expanduser()
+        if raw_path.is_absolute():
+            manifest_abs = raw_path.resolve()
+        else:
+            repo_root = find_repo_root()
+            manifest_abs = (repo_root / raw_path).resolve()
+        issues = lint_report(manifest_abs)
+        ok = not issues
+
+        response: Dict[str, Any] = {
+            "manifest_path": str(manifest_abs),
+            "ok": ok,
+            "issues": [asdict(issue) for issue in issues],
+        }
+
+        if validate_only:
+            return response
+
+        if fail_on_issues and not ok:
+            return response
+
+        result = build_report(manifest_abs, output_name=output_name, refresh=False)
+        result_format = result.get("format", "markdown")
+        body = result.get("body", "")
+        provenance = result.get("provenance", {})
+
+        effective_format = format or result_format
+        body_json = None
+        if effective_format == "html" and result_format == "markdown":
+            body = f"<html><body><pre>\n{body}\n</pre></body></html>\n"
+        elif effective_format == "json":
+            if result_format == "json":
+                try:
+                    body_json = json.loads(body)
+                except Exception:
+                    body_json = None
+            else:
+                # Wrap non-JSON bodies so callers get valid JSON when requesting it
+                body_json = {"format": result_format, "body": body}
+                body = json.dumps(body_json)
+
+        selected_output_name: Optional[str] = output_name
+        written_path: Optional[str] = None
+
+        if persist_output:
+            if output_path is not None:
+                out_path = Path(output_path).expanduser()
+                if not out_path.is_absolute():
+                    repo_root = find_repo_root()
+                    out_path = (repo_root / out_path).resolve()
+            else:
+                manifest_obj = load_manifest(manifest_abs)
+                selected = None
+                if output_name is not None:
+                    for candidate in manifest_obj.outputs:
+                        if candidate.name == output_name:
+                            selected = candidate
+                            break
+                if selected is None:
+                    if not manifest_obj.outputs:
+                        raise ValueError(
+                            "Report manifest defines no outputs and output_path was not provided",
+                        )
+                    selected = manifest_obj.outputs[0]
+                selected_output_name = selected.name
+                out_path = Path(selected.path).expanduser()
+                if not out_path.is_absolute():
+                    repo_root = find_repo_root()
+                    out_path = (repo_root / out_path).resolve()
+
+            out_path.parent.mkdir(parents=True, exist_ok=True)
+            out_path.write_text(body, encoding="utf-8")
+            written_path = str(out_path)
+
+        response.update(
+            {
+                "output_name": selected_output_name,
+                "format": effective_format,
+                "provenance": provenance,
+            }
+        )
+
+        if include_body:
+            response["body"] = body
+
+        if effective_format == "json" and include_body:
+            if body_json is None:
+                try:
+                    body_json = json.loads(body)
+                except Exception:
+                    body_json = None
+            if body_json is not None:
+                response["body_json"] = body_json
+
+        if written_path is not None:
+            response["output_path"] = written_path
+
+        return response
+
     @server.tool(
         name="record_query_insight",
         description="Attach a post-hoc insight to a prior execute_query run",
