@@ -176,6 +176,47 @@ def generate_sql_alternatives(
     return alternatives
 
 
+def _strip_leading_comments_and_whitespace(statement: str) -> str:
+    """Return statement without leading whitespace or SQL comments."""
+
+    if not statement:
+        return ""
+
+    length = len(statement)
+    idx = 0
+
+    while idx < length:
+        # Skip whitespace characters
+        while idx < length and statement[idx].isspace():
+            idx += 1
+
+        if idx >= length:
+            break
+
+        two = statement[idx : idx + 2]
+
+        # Line comment starting with --
+        if two == "--":
+            idx += 2
+            while idx < length and statement[idx] not in "\n\r":
+                idx += 1
+            while idx < length and statement[idx] in "\n\r":
+                idx += 1
+            continue
+
+        # Block comment starting with /* ... */
+        if two == "/*":
+            idx += 2
+            while idx < length and statement[idx : idx + 2] != "*/":
+                idx += 1
+            idx = min(length, idx + 2)
+            continue
+
+        break
+
+    return statement[idx:]
+
+
 def validate_sql_statement(
     statement: str,
     allow_list: List[str],
@@ -293,6 +334,25 @@ def validate_sql_statement(
     if is_valid:
         return stmt_type, True, None
 
+    # Safe lexical fallback for Snowflake SHOW statements
+    # Upstream parsers (including sqlglot) often classify SHOW as Command/Unknown.
+    # Since SHOW is read-only and already governed by the 'show' permission, we
+    # recognize it lexically to avoid false blocks.
+    try:
+        stripped = _strip_leading_comments_and_whitespace(statement)
+    except Exception:  # pragma: no cover - extremely defensive
+        stripped = statement.lstrip() if hasattr(statement, "lstrip") else statement
+
+    if stripped.upper().startswith("SHOW"):
+        # Always normalize to the "Show" statement type for clarity.
+        stmt_type = "Show"
+
+        # Disallow entries must win even if SHOW appears in allow_list.
+        if "show" in disallow_set:
+            is_valid = False
+        elif "show" in allow_set:
+            return "Show", True, None
+
     # Generate error message with alternatives
     alternatives = generate_sql_alternatives(statement, stmt_type)
 
